C-------------------------------------------------------------------
      subroutine gwpakt(xv,av,bv,xmax,xmin,m,t,h,sigma)
      IMPLICIT NONE
C     This routine plots the wave packet for a given time t and for
C     xmax > x > xmin. The wave packet fourier transform is a
C     a gaussian function.
C
C     The input data are t, h, sigma, xmin, xmax and m.
C
C     n - number of points in the wavenumber space discretization
C         power of 2.
C     h - water depth/wavelength
C     t - time/sqrt(water depth/g)
C     sigma - the widt of the wave packet spectrum.
C     m - number of points in the vector xv
C
      integer*4 j,nmax,dmax,nk,m
      parameter(nmax=200000,dmax=100000)
      real*8 omega,b,h,xv(*),av(*),fv(nmax),kv(nmax),
     *     y2(nmax+1),dx,t,kmax,xcl(nmax),xmax,xmin,
     *     x,lbmax,aux,auxa,dk,sigma,lx,
     *     data(2*nmax),bv(*),cgmax
C
      integer n
      real*8 o,zr,a1,a2,a4,a9,g,pi,a49,tol
      parameter(o=1.0d+00,zr=0.0d+00,a1=2.0d+00,a2=3.0d+00,
     *     g=9.8d+00,pi=3.141592653589793238d+00,a4=5.0d+00,
     *     a9=10.0d+00,a49=50.0d+00,n=128)
C
      integer ipwer2
      real*8 funha
C
C     maximum group velocity 
C
      cgmax=h
C
C     quantities dx,dk,kmax and space and wavenumber discretization
C
      lx=xmax+cgmax*t
      nk=n
C     
      dx=(xmax-xmin)/dble(m-1)
      kmax=a49
C     write(*,*) 'lx,cgmax,kmax,dx',lx,cgmax,kmax,dx
C     
      call gndprm(t,cgmax,lx,dx,dk,kmax,nk,xcl,kv)
C     
C     function in the wavenumber space evaluation
C     
      call gndata(data,kv,lx,nk,sigma,t,h,dk)
C     
C     inverse cossine transform
C     
      call four1(data,nk,-1)
C     
C     generate output from the transform output
C     
      do j=1,nk,1
         fv(j)=data(2*j-1)/(a1*pi)
      end do
C     
      call spline(xcl,fv,nk,zr,zr,y2,dmax)
C     
      do j=1,m,1
         call splint(xcl,fv,y2,nk,xv(j),av(j))
         bv(j)=funha(xv(j),sigma,h,t)
      end do
C     
      return
      end
C-------------------------------------------------------------------
      subroutine gndprm(t,cg,lx,dx,dk,kmax,n,xv,kv)
      IMPLICIT NONE
C     evaluate the parameters dx,dk,kmax
C     t - time to consider
C     cg - group velocity.
C     n - number of points in the k discretization
C
      integer n
      real*8 t,cg,dx,dk,kmax,lx,xv(*),kv(*)
C
      integer i,na
      real*8 zr,o,a1,pi
      parameter(zr=0.0d+00,o=1.0d+00,a1=2.0d+00,
     $     pi=3.141592653589793238d+00)
C
      integer ipwer2
C
C     space steps
C
      dx=dmin1((a1*lx)/dble(n),dx,pi/kmax)
C
C     redefine n if necessary
C
      na=int(sngl(((a1*lx)/dx)))+1
      if(na.gt.n) then
         n=ipwer2(na)
         dx=(a1*lx)/dble(n)
      endif         
C
C     define kmax 
C
      kmax=(a1*pi)/(a1*dx)
C
      dk=(a1*kmax)/dble(n)
C
      if(t.gt.zr) then
C
C     redefine dk if necessary
C
         dk=dmin1((a1*pi)/(cg*t),dk)
C
         n=int(sngl((a1*kmax/dk)))-1
C
         n=ipwer2(n)
C
         dx=(a1*lx)/dble(n)
C
         kmax=(a1*pi)/(a1*dx)
C
         dk=(a1*kmax)/dble(n)
C
      endif
C
      do i=1,n,1
         xv(i)=-lx+dble(i-1)*dx
      end do
C
      do i=1,n/2+1,1
         kv(i)=dk*dble(i-1)
      end do
C
      do i=n/2+1,n-1,1
         kv(i+1)=-dble(-i+n)*dk
      end do
C
      return
      end
C-------------------------------------------------------------------
      integer function ipwer2(n)
      IMPLICIT NONE
C     This function gives the integer 2^m >= n.
C     n - imput (given natural number)/output (number 2^m >= n)
C
      integer n
C
      integer i,j
C
      i=2
      j=1
C
      do while(i.lt.n)
         i=2**j
         j=j+1
      end do
C
      ipwer2=i
C
      return
      end
C-------------------------------------------------------------------
      subroutine gndata(data,kv,lx,n,sigma,t,h,dk)
      IMPLICIT NONE
C     Generates the vector data as input for the two-dimensional
C     fourier transform.
C
C     lx - half size of the domain in x
C     n - number of points in the k discretization
C     pwn - carrier wave wavenumber
C     sigma - standard deviation of the Gaussian 
C     dx - space step.
C     dk - wavenumber step.
C
      integer n
      real*8 lx,pwn,sigma,t,h,dk
      real*8 data(*),kv(*)
C
      integer i1
      real*8 zr,a1,pi,wn,auxa
      parameter(zr=0.0d+00,a1=2.0d+00,pi=3.141592653589793238d+00)
      complex*16 zaux
C
      real*8 funh
C
      pwn=a1*pi
C
      do i1=1,n,1
         wn=dabs(kv(i1))
C
         auxa=funh(wn,pwn,sigma,t,h)
C
         zaux=dcmplx(dk*auxa,zr)*zexp(dcmplx(zr,wn*lx))
C     
         data(2*i1-1)=dble(zaux)
         data(2*i1)=dimag(zaux)
C
      end do
C
      return
      end
C-------------------------------------------------------------------
      real*8 function funh(p,kt,sigma,t,h)
      IMPLICIT NONE
C     This function evaluates expression
C
C     (amp/(sqrt(2*pi*sigma)))*(exp(-(k-kt)^2/(2*sigma^2))
C     +exp(-(k+kt)^2/(2*sigma^2)))*cos(omega(k)*t)
C
C     where omega(*) is the dispersion relation.
C     w - excitation frequency
C
      real*8 p,kt,x,t,h,sigma,tol
C
      real*8 omega
      real*8 zr,o,a1,a2,a3,pi,auxa,auxb,auxc,auxd,auxe
      parameter(zr=0.0d+00,o=1.0d+00,a1=2.0d+00,a3=4.0d+00,
     $     pi=3.141592653589793238d+00)
C
      real*8 dcosm
C
      auxa=p*h
      auxa=dtanh(auxa)
      auxa=p*h*auxa
      omega=dsqrt(auxa)
      auxa=omega*t
      auxa=dcosm(auxa)
C
      auxb=p-kt
      auxb=auxb**2
      auxb=auxb/(a1*sigma)
      auxb=dexp(-auxb)
C
      auxc=p+kt
      auxc=auxc**2
      auxc=auxc/(a1*sigma)
      auxc=dexp(-auxc)
C
      auxd=o/(a1*dsqrt(a1*pi*sigma))
C
      auxe=auxd*(auxb+auxc)*auxa
C
      funh=auxe
C
      return
      end
C-------------------------------------------------------------------
      real*8 function funha(x,sigma,h,t)
      IMPLICIT NONE
C     This function evaluates expression
C
C     (amp/(4*pi))*(exp(-((x-sqrt(g*h)*t)^2)*sigma/2)*
C     cos(kt*(x-sqrt(g*h)*t))
C     +exp(-((x+sqrt(g*h)*t)^2)*sigma/2)*cos(kt*(x+sqrt(g*h)*t)))
C
C     where omega(*) is the dispersion relation.
C     w - excitation frequency
C
      real*8 kt,x,t,h,sigma
C
      real*8 zr,o,a1,a2,a3,pi,auxa,auxb,auxc,auxd,auxe,auxf,
     $     auxg,auxh
      parameter(zr=0.0d+00,o=1.0d+00,a1=2.0d+00,a3=4.0d+00,
     $     pi=3.141592653589793238d+00)
C
      real*8 dcosm
C
      kt=a1*pi
C
      auxa=x-dsqrt(kt*h*dtanh(kt*h))*t
      auxb=x+dsqrt(kt*h*dtanh(kt*h))*t
      auxc=(auxa**2)*sigma/a1
      auxd=(auxb**2)*sigma/a1
      auxe=kt*auxa
      auxf=kt*auxb
C
      auxg=o/(a3*pi)
C
      auxh=auxg*(dexp(-auxc)*dcosm(auxe)+dexp(-auxd)*dcosm(auxf))
C
      funha=auxh
C
      return
      end
C-------------------------------------------------------------------
      real*8 function dcosm(x)
      IMPLICIT NONE
C     This function evaluates the cossine function in double precission
C
      real*8 x
C
      integer n
      real*8 a1,pi,aux
C
      parameter(a1=2.0d+00,pi=3.141592653589793238d+00)
C
      if(x.le.a1*pi) then
         dcosm=dcos(x)
      else
         n=idnint(x/(a1*pi))
         aux=a1*pi*dble(n)
         if(aux.gt.x) then
            n=n-1
            aux=a1*pi*dble(n)
         endif
         aux=x-aux
         dcosm=dcos(aux)
      endif
C
      return
      end
C-------------------------------------------------------------------
      real*8 function dsinm(x)
      IMPLICIT NONE
C     This function evaluates the sine function in double precission
C
      real*8 x
C
      integer n
      real*8 a1,pi,aux
C
      parameter(a1=2.0d+00,pi=3.141592653589793238d+00)
C
      if(x.le.a1*pi) then
         dsinm=dsin(x)
      else
         n=idnint(x/(a1*pi))
         aux=a1*pi*dble(n)
         if(aux.gt.x) then
            n=n-1
            aux=a1*pi*dble(n)
         endif
         aux=x-aux
         dsinm=dsin(aux)
      endif
C
      return
      end
C--------------------------------------------------------------------
      SUBROUTINE four1(data,nn,isign)
      IMPLICIT NONE
C
      INTEGER isign,nn
      REAL*8 data(*)
      INTEGER i,istep,j,m,mmax,n
      REAL*8 tempi,tempr
      DOUBLE PRECISION theta,wi,wpi,wpr,wr,wtemp
C
      real*8 dsinm
C
      n=2*nn
      j=1
      do 11 i=1,n,2
        if(j.gt.i)then
          tempr=data(j)
          tempi=data(j+1)
          data(j)=data(i)
          data(j+1)=data(i+1)
          data(i)=tempr
          data(i+1)=tempi
        endif
        m=n/2
1       if ((m.ge.2).and.(j.gt.m)) then
          j=j-m
          m=m/2
        goto 1
        endif
        j=j+m
11    continue
      mmax=2
2     if (n.gt.mmax) then
        istep=2*mmax
        theta=6.28318530717959d0/(isign*mmax)
        wpr=-2.d0*dsinm(0.5d0*theta)**2
        wpi=dsinm(theta)
        wr=1.d0
        wi=0.d0
        do 13 m=1,mmax,2
          do 12 i=m,n,istep
            j=i+mmax
            tempr=wr*data(j)-wi*data(j+1)
            tempi=wr*data(j+1)+wi*data(j)
            data(j)=data(i)-tempr
            data(j+1)=data(i+1)-tempi
            data(i)=data(i)+tempr
            data(i+1)=data(i+1)+tempi
12        continue
          wtemp=wr
          wr=wr*wpr-wi*wpi+wr
          wi=wi*wpr+wtemp*wpi+wi
13      continue
        mmax=istep
      goto 2
      endif
      return
      END
C--------------------------------------------------------------------
      SUBROUTINE spline(x,y,n,yp1,ypn,y2,dmax)
      IMPLICIT NONE
C     Given arrays x(1:n) and y(1:n) containing a tabulated function,
C     i.e. yi=f(xi), with x1<x2<...<xn, and given values yp1 and ypn 
C     for the first derivative of the interpolating function at points
C     1 and n, respectively, this routine returns an array y2(1:n) of
C     points xi. If yp1 and/or ypn are equal to 10E+30 or larder, the
C     routine is signaled to set the corresponding boundary condition
C     for a natural spline, with zero second derivative on that 
C     boundary.
C
      INTEGER n,dmax
      REAL*8 yp1,ypn,x(*),y(*),y2(*)
      INTEGER i,k
      REAL*8 p,qn,sig,un,u(dmax+1)
c      write(*,*) 'in spline'
      if (yp1.gt..99e30) then
        y2(1)=0.0d+00
        u(1)=0.0d+00
      else
        y2(1)=-0.5d+00
        u(1)=(3.0d+00/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.0d+00
        y2(i)=(sig-1.0d+00)/p
        u(i)=(6.0d+00*((y(i+1)-y(i))/(x(i+
     *1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*
     *u(i-1))/p
11    continue
      if (ypn.gt..99e30) then
        qn=0.0d+00
        un=0.0d+00
      else
        qn=0.5d+00
        un=(3.0d+00/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.0d+00)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
c      write(*,*) 'out spline'
      return
      END
C----------------------------------------------------------------------
      SUBROUTINE splint(xa,ya,y2a,n,x,y)
      IMPLICIT NONE
C
C     Given arrays xa(1:n) and ya(1:n) of length n, which tabulate a 
C     function (with the xai's in order), and given the array 
C     y2a(1:n), which is the output from spline above, and given a 
C     value of x, this routine returns a cubic-spline interpolated 
C     value y.
C
      INTEGER n
      REAL*8 x,y,xa(*),y2a(*),ya(*)
      INTEGER k,khi,klo
      REAL*8 a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) pause 'bad xa input in splint'
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*
     *     (h**2)/6.0d+00
      return
      END
C-----------------------------------------------------------------------



